@@ -1,5 +1,3 @@
-use core::convert::{TryFrom, TryInto};
-
 use coap_message::{MessageOption, MutableWritableMessage, ReadableMessage};
 
 use coap_numbers::{code, option};
@@ -33,13 +31,20 @@ pub struct Block2RequestData {
     szx: u8,
 }
 
+/// Error that occurs when constructing a `Block2RequestData` from a message or an option.
+///
+/// It is singular and contains no details (for there is no usable action from them), but usually
+/// stems from either the option being repeated or having an excessively large value.
+#[derive(Debug)]
+pub struct BadBlock2Option;
+
 impl Block2RequestData {
     /// Extract a request block 2 value from a request message.
     ///
     /// Absence of the option is not an error and results in the default value to be returned;
     /// exceeding length or duplicate entries are an error and are indicated by returning an error,
     /// which should be responded to with a Bad Option error.
-    pub fn from_message(message: &impl ReadableMessage) -> Result<Self, ()> {
+    pub fn from_message(message: &impl ReadableMessage) -> Result<Self, BadBlock2Option> {
         let mut b2options = message.options().filter(|o| o.number() == option::BLOCK2);
 
         match b2options.next() {
@@ -48,7 +53,7 @@ impl Block2RequestData {
                 if b2options.next().is_none() {
                     Self::from_option(&o)
                 } else {
-                    Err(())
+                    Err(BadBlock2Option)
                 }
             }
         }
@@ -64,12 +69,12 @@ impl Block2RequestData {
     /// # Panics
     ///
     /// In debug mode if the option is not Block2
-    pub fn from_option(option: &impl MessageOption) -> Result<Self, ()> {
+    pub fn from_option(option: &impl MessageOption) -> Result<Self, BadBlock2Option> {
         debug_assert!(option.number() == option::BLOCK2);
-        let o: u32 = option.value_uint().ok_or(())?;
+        let o: u32 = option.value_uint().ok_or(BadBlock2Option)?;
         // Block2 is up to 3 long
         if o >= 0x1000000 {
-            return Err(());
+            return Err(BadBlock2Option);
         }
         Ok(Self {
             szx: (o as u8) & 0x7,
@@ -77,6 +82,10 @@ impl Block2RequestData {
         })
     }
 
+    pub fn to_option_value(&self, more: bool) -> u32 {
+        (self.blknum << 4) | if more { 0x08 } else { 0 } | self.szx as u32
+    }
+
     /// Size of a single block
     pub fn size(&self) -> u16 {
         1 << (4 + self.szx)
@@ -166,21 +175,23 @@ where
             response.add_option_uint(cfopt, cf);
         }
     }
-    response.add_option_uint(
-        optconvert(option::BLOCK2),
-        (block2.blknum << 4) | block2.szx as u32,
-    );
+    response.add_option_uint(optconvert(option::BLOCK2), block2.to_option_value(false));
 
     let (characterization, written, etag, ret) = {
         let full_payload = response.payload_mut_with_len(block2.size().into());
         let mut writer = WindowedInfinityWithETag::new(
             &mut full_payload[..block2.size() as usize],
-            -(block2.before() as isize)
+            -(block2.before() as isize),
         );
 
         let ret = f(&mut writer);
 
-        (writer.characterize_cursor(), writer.buffer.get_written().len(), writer.etag(), ret)
+        (
+            writer.characterize_cursor(),
+            writer.buffer.get_written().len(),
+            writer.etag(),
+            ret,
+        )
     };
 
     response.truncate(written);
@@ -231,19 +242,11 @@ impl<'a> WindowedInfinityWithETag<'a> {
         self.buffer.write(s);
     }
 
-    #[deprecated(note = "Renamed to write for consistency with WindowedInfinity")]
-    pub fn write_bytes(&mut self, s: &[u8]) {
-        self.write(s)
-    }
-
     pub fn etag(&self) -> [u8; 8] {
         self.etag.to_le_bytes()
     }
 }
 
-#[deprecated(note = "Renamed to WindowedInfinityWithETag")]
-pub type BlockWriter<'a> = WindowedInfinityWithETag<'a>;
-
 impl<'a> ::core::fmt::Write for WindowedInfinityWithETag<'a> {
     fn write_str(&mut self, s: &str) -> ::core::fmt::Result {
         self.write(s.as_bytes());
@@ -263,12 +266,18 @@ impl<'a> serde_cbor::ser::Write for WindowedInfinityWithETag<'a> {
 ///
 /// This is used by a [crate::HandlerBuilder] (in particular, its path-based [crate::ForkingHandler]) to free the
 /// resources from the strange duty of skipping over a critical option they are unaware of.
+// TBD: Consider removing this in favor of MaskingUriUpToPathN -- if both are used, the flash
+// consumption of having both surely outweighs the runtime overhead of decrementing while going
+// through the options, and the length is known in advance anyway.
 pub struct MaskingUriUpToPath<'m, M: ReadableMessage>(pub &'m M);
 
 impl<'m, M: ReadableMessage> ReadableMessage for MaskingUriUpToPath<'m, M> {
     type Code = M::Code;
     type MessageOption<'a> = M::MessageOption<'a>;
-    type OptionsIter<'a> = MaskingUriUpToPathIter<M::OptionsIter<'a>>;
+    type OptionsIter<'a>
+    where
+        Self: 'a,
+    = MaskingUriUpToPathIter<M::OptionsIter<'a>>;
 
     fn options(&self) -> Self::OptionsIter<'_> {
         MaskingUriUpToPathIter(self.0.options())
@@ -299,3 +308,66 @@ impl<MO: MessageOption, I: Iterator<Item = MO>> Iterator for MaskingUriUpToPathI
         }
     }
 }
+
+/// Like [MaskingUriUpToPath], but only consuming a given number of Uri-Path options -- suitable
+/// for ForkingTreeHandler.
+pub(crate) struct MaskingUriUpToPathN<'m, M: ReadableMessage> {
+    message: &'m M,
+    strip_paths: usize,
+}
+
+impl<'m, M: ReadableMessage> MaskingUriUpToPathN<'m, M> {
+    pub(crate) fn new(message: &'m M, strip_paths: usize) -> Self {
+        Self {
+            message,
+            strip_paths,
+        }
+    }
+}
+
+impl<'m, M: ReadableMessage> ReadableMessage for MaskingUriUpToPathN<'m, M> {
+    type Code = M::Code;
+    type MessageOption<'a> = M::MessageOption<'a>;
+    type OptionsIter<'a>
+    where
+        Self: 'a,
+    = MaskingUriUpToPathNIter<M::OptionsIter<'a>>;
+
+    fn options(&self) -> Self::OptionsIter<'_> {
+        MaskingUriUpToPathNIter {
+            inner: self.message.options(),
+            remaining_strip: self.strip_paths,
+        }
+    }
+
+    fn code(&self) -> M::Code {
+        self.message.code()
+    }
+
+    fn payload(&self) -> &[u8] {
+        self.message.payload()
+    }
+}
+
+pub struct MaskingUriUpToPathNIter<I> {
+    inner: I,
+    remaining_strip: usize,
+}
+
+impl<MO: MessageOption, I: Iterator<Item = MO>> Iterator for MaskingUriUpToPathNIter<I> {
+    type Item = MO;
+
+    fn next(&mut self) -> Option<MO> {
+        loop {
+            let result = self.inner.next()?;
+            match result.number() {
+                coap_numbers::option::URI_HOST => continue,
+                coap_numbers::option::URI_PATH if self.remaining_strip > 0 => {
+                    self.remaining_strip -= 1;
+                    continue;
+                }
+                _ => return Some(result),
+            }
+        }
+    }
+}
