@@ -280,16 +280,22 @@ where
     // (After that, could be that it's warranted as it's now more than a few 4-liners, could be
     // that not).
 
-    type Record = ForkingRecord<PrefixedRecord<'a, H1::Record>, H2::Record>;
+    type Record<'b>
+    where
+        Self: 'b,
+    = ForkingRecord<PrefixedRecord<'b, H1::Record<'b>>, H2::Record<'b>>;
 
-    type Reporter = core::iter::Chain<
-        core::iter::Map<H2::Reporter, fn(H2::Record) -> Self::Record>,
+    type Reporter<'b>
+    where
+        Self: 'b,
+    = core::iter::Chain<
+        core::iter::Map<H2::Reporter<'b>, fn(H2::Record<'b>) -> Self::Record<'b>>,
         core::iter::Map<
-            core::iter::Zip<H1::Reporter, core::iter::Repeat<&'a [&'a str]>>,
-            fn((H1::Record, &'a [&'a str])) -> Self::Record,
+            core::iter::Zip<H1::Reporter<'b>, core::iter::Repeat<&'b [&'b str]>>,
+            fn((H1::Record<'b>, &'b [&'b str])) -> Self::Record<'b>,
         >,
     >;
-    fn report(&self) -> Self::Reporter {
+    fn report(&self) -> Self::Reporter<'_> {
         fn first<'c, H1R, H2R>(
             prefixed_prefix: (H1R, &'c [&'c str]),
         ) -> ForkingRecord<PrefixedRecord<'c, H1R>, H2R> {
@@ -365,14 +371,20 @@ where
     H1: Handler<RequestData = RD1> + wkc::Reporting,
     H2: Handler<RequestData = RD2> + wkc::Reporting,
 {
-    type Record = ForkingRecord<PrefixedRecord<'a, H1::Record>, H2::Record>;
+    type Record<'b>
+    where
+        Self: 'b,
+    = ForkingRecord<PrefixedRecord<'b, H1::Record<'b>>, H2::Record<'b>>;
 
     // FIXME: one of these is copied over to wkc::Reporting for ForkingHandler, see there.
 
     #[cfg(feature = "nontrivial_option_processing")]
-    type Reporter = impl Iterator<Item = Self::Record>;
+    type Reporter<'b>
+    where
+        Self: 'b,
+    = impl Iterator<Item = Self::Record<'b>>;
     #[cfg(feature = "nontrivial_option_processing")]
-    fn report(&self) -> Self::Reporter {
+    fn report(&self) -> Self::Reporter<'_> {
         self.h2
             .report()
             .map(ForkingRecord::Second as fn(_) -> _)
@@ -385,15 +397,18 @@ where
     }
 
     #[cfg(not(feature = "nontrivial_option_processing"))]
-    type Reporter = core::iter::Chain<
-        core::iter::Map<H2::Reporter, fn(H2::Record) -> Self::Record>,
+    type Reporter<'b>
+    where
+        Self: 'b,
+    = core::iter::Chain<
+        core::iter::Map<H2::Reporter<'b>, fn(H2::Record<'b>) -> Self::Record<'b>>,
         core::iter::Map<
-            core::iter::Zip<H1::Reporter, core::iter::Repeat<&'a [&'a str]>>,
-            fn((H1::Record, &'a [&'a str])) -> Self::Record,
+            core::iter::Zip<H1::Reporter<'b>, core::iter::Repeat<&'b [&'b str]>>,
+            fn((H1::Record<'b>, &'b [&'b str])) -> Self::Record<'b>,
         >,
     >;
     #[cfg(not(feature = "nontrivial_option_processing"))]
-    fn report(&self) -> Self::Reporter {
+    fn report(&self) -> Self::Reporter<'_> {
         fn first<'c, H1R, H2R>(
             prefixed_prefix: (H1R, &'c [&'c str]),
         ) -> ForkingRecord<PrefixedRecord<'c, H1R>, H2R> {
@@ -527,10 +542,16 @@ impl<T> wkc::Reporting for SimpleRendered<T>
 where
     T: SimpleRenderable,
 {
-    type Record = wkc::EmptyRecord;
-    type Reporter = core::iter::Once<wkc::EmptyRecord>;
+    type Record<'a>
+    where
+        Self: 'a,
+    = wkc::EmptyRecord;
+    type Reporter<'a>
+    where
+        Self: 'a,
+    = core::iter::Once<wkc::EmptyRecord>;
 
-    fn report(&self) -> Self::Reporter {
+    fn report(&self) -> Self::Reporter<'_> {
         // Using a ConstantSliceRecord instead would be tempting, but that'd need a const return
         // value from self.0.content_format()
         core::iter::once(wkc::EmptyRecord {})
